import numpy as np
import sys
import math
import caffe
import random
from PIL import Image
from fine_training import *


sys.path.insert(0, CAFFE_root + 'python')


class DataLayer(caffe.Layer):
    def setup(self, bottom, top):
        self.random = True
        self.seed = 1337
        image_list = open(training_set_filename(current_fold), 'r').read().splitlines()
        self.training_image_set = np.zeros((len(image_list)), dtype = np.int)
        for i in range(len(image_list)):
            s = image_list[i].split(' ')
            self.training_image_set[i] = int(s[0])
        slice_list = open(list_training[plane], 'r').read().splitlines()
        self.image_ID = np.zeros((len(slice_list)), dtype = np.int)
        self.image_filename = ['' for l in range(len(slice_list))]
        self.label_filename = ['' for l in range(len(slice_list))]
        self.average = np.zeros((len(slice_list)))
        self.pixels = np.zeros((len(slice_list)), dtype = np.int)
        self.minA = np.zeros((len(slice_list)), dtype = np.int)
        self.maxA = np.zeros((len(slice_list)), dtype = np.int)
        self.minB = np.zeros((len(slice_list)), dtype = np.int)
        self.maxB = np.zeros((len(slice_list)), dtype = np.int)
        for l in range(len(slice_list)):
            s = slice_list[l].split(' ')
            self.image_ID[l] = s[0]
            self.image_filename[l] = s[2]
            self.label_filename[l] = s[3]
            self.average[l] = float(s[4])
            self.pixels[l] = int(s[organ_ID * 5])
            self.minA[l] = int(s[organ_ID * 5 + 1])
            self.maxA[l] = int(s[organ_ID * 5 + 2])
            self.minB[l] = int(s[organ_ID * 5 + 3])
            self.maxB[l] = int(s[organ_ID * 5 + 4])
        if slice_threshold <= 1:
            pixels_index = sorted(range(len(slice_list)), key = lambda l: self.pixels[l])
            last_index = int(math.floor((self.pixels > 0).sum() * slice_threshold))
            min_pixels = self.pixels[pixels_index[-last_index]]
        else:
            min_pixels = slice_threshold
        self.active_index = [l for l, p in enumerate(self.pixels) if p >= min_pixels]
        if len(top) != 2:
            raise Exception('Need to define two tops: data and label.')
        if len(bottom) != 0:
            raise Exception('Do not define a bottom.')
        self.index = -1
        if self.random:
            random.seed(self.seed)
        self.next_slice_index()


    def next_slice_index(self):
        while True:
            if self.random:
                self.index = random.randint(0, len(self.active_index) - 1)
            else:
                self.index += 1
                if self.index == len(self.active_index):
                    self.index = 0
            if self.image_ID[self.active_index[self.index]] in self.training_image_set:
                break
        self.left = random.randint(margin_mean - margin_random, margin_mean + margin_random)
        self.right = random.randint(margin_mean - margin_random, margin_mean + margin_random)
        self.top = random.randint(margin_mean - margin_random, margin_mean + margin_random)
        self.bottom = random.randint(margin_mean - margin_random, margin_mean + margin_random)


    def reshape(self, bottom, top):
        self.data = self.load_image(self.index)
        self.label = self.load_label(self.index)
        top[0].reshape(1, *self.data.shape)
        top[1].reshape(1, *self.label.shape)


    def forward(self, bottom, top):
        top[0].data[...] = self.data
        top[1].data[...] = self.label
        self.next_slice_index()


    def backward(self, top, propagate_down, bottom):
        pass


    def load_image(self, index):
        slice_ID = self.active_index[self.index]
        image_ = np.load(self.image_filename[slice_ID])
        image_ = image_[max(self.minA[slice_ID] - self.left, 0): \
            self.maxA[slice_ID] + self.right + 1, \
            max(self.minB[slice_ID] - self.top, 0): self.maxB[slice_ID] + self.bottom + 1]
        image_ = image_.astype(np.float32)
        image_[image_ < low_range] = low_range
        image_[image_ > high_range] = high_range
        image_ = (image_ - low_range) / (high_range - low_range)
        data = image_.reshape((1, image_.shape[0], image_.shape[1]))
        data = np.repeat(data, 3, axis = 0)
        return data


    def load_label(self, index):
        slice_ID = self.active_index[self.index]
        volume = np.load(self.label_filename[slice_ID])
        volume = volume[max(self.minA[slice_ID] - self.left, 0): \
            self.maxA[slice_ID] + self.right + 1, \
            max(self.minB[slice_ID] - self.top, 0): self.maxB[slice_ID] + self.bottom + 1]
        volume[volume <> organ_ID] = 0
        volume[volume == organ_ID] = 1
        label = np.array(volume, dtype = np.uint8)
        label = label.reshape((1, volume.shape[0], volume.shape[1]))
        return label
